﻿package MT::Plugin::Pager;
use strict;
use MT;
use MT::Plugin;
use MT::Template::Context;
use MT::WeblogPublisher;
use MT::FileInfo;
use File::Basename;

our $VERSION = '0.5';
our $SCHEMA_VERSION = '0.01';

use base qw( MT::Plugin );

@MT::Plugin::Pager::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::Pager({
    name => 'Pager',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    version     => $VERSION,
    schema_version => $SCHEMA_VERSION,
    l10n_class => 'Pager::L10N',
});

MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        callbacks => {
            'build_file_filter'
                 => \&_build_file_filter,
            'MT::App::CMS::template_param.edit_template'
                 => \&_edit_template,
        },
        object_types => {
            'template' => {
                'pager' => 'integer',
            },
        },
        tags => {
            block => {
                'Pager' => \&_pager_loop,
                'IfPagerPrev?'   => \&_if_prev,
                'IfPagerNext?'   => \&_if_next,
                'IfPagerHeader?' => \&_if_header,
                'IfPagerFooter?' => \&_if_footer,
                'IfPagerCurrent?' => \&_if_current,
            },
            function => {
                'PagerTotal'    => \&_pager_total,
                'PagerLink'     => \&_pager_link,
                'PagerCounter'  => \&_pager_counter,
                'PagerNextLink' => \&_pager_next,
                'PagerPrevLink' => \&_pager_prev,
                'PagerCurrent'  => \&_pager_current,
            },
        },
   });
}

sub _build_file_filter {
    my ( $eh, %args ) = @_;
    my $template = $args{'Template'};
    my $pager = $template->pager;
    if ( $pager ) {
        my $app   = MT->instance();
        my $tmpl_id = $template->id;
        my $ctx   = $args{'Context'};
        my $file  = $args{'File'};
        my $blog  = $args{'Blog'};
        my $map   = $args{'TemplateMap'};
        my $at    = $args{'ArchiveType'};
        my $start = $args{'PeriodStart'};
        my $category = $args{'Category'};
        my $orig_finfo = $args{'FileInfo'};
        my $mt   = MT::WeblogPublisher->new;
        my $archiver = $mt->archiver( $at );
        unless ( $at eq 'index' ) {
			if ( $archiver->archive_group_entries ) {
				my $entries = $archiver->archive_group_entries->($ctx);
				$ctx->stash( 'entries', $entries );
			}
        }
        $ctx->stash( 'blog', $blog );
        my $arch_root = ( $at eq 'Page' ) ? $blog->site_path : $blog->archive_path;
        my $build = MT::Builder->new;
        my $count;
        unless ( $at eq 'index' ) {
            $count = MT::Template::Context::_hdlr_archive_count($ctx);
        } else {
            $count = MT::Entry->count({ blog_id => $blog->id,
                                       status  => MT::Entry::RELEASE(),
                                    });
            $ctx->stash( 'at_index', 1 );
        }
        $ctx->stash( 'total', $count );
        my $tmpl_src = $template->text;
        my $limit = $1 if $tmpl_src =~ /<mt:*entries.*?limit=.*?([0-9]{1,}).*?>/is;
        my $repeat = 1;
        if ( $limit < $count ) {
            $repeat = $count / $limit;
            $repeat = ceil ( $repeat );
        }
        $repeat--;
        if ( $repeat ) {
            my $fmgr = $blog->file_mgr;
            my $template = $template->clone();
            $template->pager( 0 );
            $args{'Template'} = $template;
            $args{'template'} = $template;
            for ( 1 .. $repeat ) {
                if ( $app->run_callbacks( 'build_file_filter', %args ) ) {
                    $ctx->{__stash}{'prev'} = 1;
                    if ( $_ != $repeat ) {
                        $ctx->{__stash}{'next'} = 1;
                    } else {
                        $ctx->{__stash}{'next'} = 0;
                    }
                    $ctx->{__stash}{'pager'} = $_ + 1;
                    my $offset = $limit * $_;
                    my $new_file = $file;
                    $tmpl_src =~ s/(<mt:*entries.*?offset=.*?)[0-9]{1,}(.*?>)/$1$offset$2/gis;
                    $template->text( $tmpl_src );
                    my $file_num = $_ + 1;
                    $new_file =~ s/(.*)(\..*)/$1_$file_num$2/;
                    my ( $cond );
                    my $html = $template->build( $ctx, $cond );
                    my $url = $blog->archive_url;
                    $url = $blog->site_url;
                    $url .= '/' unless $url =~ m|/$|;
                    $url .= $map->{__saved_output_file};
                    $url =~ s/(.*)(\..*)/$1_$file_num$2/;
                    my ( $rel_url ) = ( $url =~ m|^(?:[^:]*\:\/\/)?[^/]*(.*)| );
                    $rel_url =~ s|//+|/|g;
                    my %terms;
                    $terms{blog_id}     = $blog->id;
                    $terms{startdate}   = $start;
                    $terms{archive_type}   = $at;
                    my ( $map_id, $category_id, $author_id );
                    unless ( $at eq 'index' ) {
                        $map_id = $map->id;
                        $category_id = $category->id if defined $category;
                        $author_id = $orig_finfo->author_id;
                        $terms{templatemap_id} = $map_id;
						$terms{category_id} = $category_id;
						$terms{author_id}   = $author_id;
                    }
                    my @finfos = MT::FileInfo->load( \%terms );
                    my $finfo;
                    if ( ( scalar @finfos == 1 )
                        && ( $finfos[0]->file_path eq $new_file )
                        && ( ( $finfos[0]->url || '' ) eq $rel_url )
                        && ( $finfos[0]->template_id == $tmpl_id ) ) {
                        $finfo = $finfos[0];
                    } else {
                        foreach ( @finfos ) { $_->remove(); }
                        $finfo = MT::FileInfo->set_info_for_url(
                            $rel_url, $new_file, $at,
                            {
                                Blog        => $blog->id,
                                TemplateMap => $map_id,
                                Template    => $tmpl_id,
                                StartDate   => $start,
                                Category    => $category_id,
                                Author      => $author_id,
                            }
                          )
                          || die "Couldn't create FileInfo because "
                          . MT::FileInfo->errstr();
                    }
                    my $orig_html = $html;
                    $args{'File'} = $new_file;
                    $args{'file'} = $new_file;
                    $args{'Content'} = \$html;
                    $args{'content'} = \$html;
                    $args{'BuildResult'} = \$orig_html;
                    $args{'build_result'} = \$orig_html;
                    $args{'RawContent'} = \$orig_html;
                    $args{'raw_content'} = \$orig_html;
                    $app->run_callbacks( 'build_page', %args );
                    if ( $fmgr->content_is_updated( $new_file, \$html ) ) {
                        require File::Spec;
                        my $path = dirname( $new_file );
                        $path =~ s!/$!!
                          unless $path eq '/';
                        unless ( $fmgr->exists( $path ) ) {
                            $fmgr->mkpath( $path )
                              or return $mt->trans_error( "Error making path '[_1]': [_2]",
                                $path, $fmgr->errstr );
                        }
                        $fmgr->put_data( $html, "$new_file.new" );
                        $fmgr->rename( "$new_file.new", $new_file );
                        $app->run_callbacks( 'build_file', %args );
                    }
                }
            }
        }
        $ctx->{__stash}{'pager'} = 0;
        $ctx->{__stash}{'prev'} = 0;
    }
    return 1;
}

sub _pager_loop {
    my ( $ctx, $args, $cond ) = @_;
    use MT::Template::Context;
    my $total = $ctx->stash( 'total' );
    my $blog = $ctx->stash( 'blog' );
    my $template = $ctx->stash( 'template' );
    my $is_pager = $ctx->stash( 'pager' );
    my $at_index = $ctx->stash( 'at_index' );
    my $tmpl_src = $template->text;
    my $limit = $1 if $tmpl_src =~ /<mt:*entries.*?limit=.*?([0-9]{1,}).*?>/is;
    my $repeat = 1;
    if ( $limit < $total ) {
        $repeat = $total / $limit;
        $repeat = ceil ( $repeat );
    }
    $repeat--;
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    my $pager_link;
    if ( $at_index ) {
        $pager_link = $blog->site_url;
        $pager_link .= '/' if ( $pager_link !~ /\/$/ );
        $pager_link .= $template->outfile;
    } else {
        $pager_link = MT::Template::Context::_hdlr_archive_link( $ctx, $cond );
    }
    if ( $pager_link =~ /(.*)\/$/ ) {
        $args->{extension} = 1;
        $pager_link .= MT::Template::Context::_hdlr_index_basename( $ctx, $args, $cond );
    }
    my $res = ''; my $i = 1;
    if ( $repeat > 1 ) {
        local $ctx->{__stash}{'next'} = 1 unless $is_pager;
        local $ctx->{__stash}{'current'} = 1 unless $is_pager;
        local $ctx->{__stash}{'current'} = $is_pager if $is_pager;
        local $ctx->{__stash}{'prev_link'} = $pager_link;
        my $prev_link = $pager_link;
        my $prev_num = $is_pager - 1;
        $prev_link =~ s/(.*)(\..*)/$1_$prev_num$2/ if ( $prev_num > 0 );
        local $ctx->{__stash}{'prev_link'} = $prev_link if ( $prev_num > 1 );
        my $next_link = $pager_link;
        $next_link =~ s/(.*)(\..*)/$1_2$2/ unless $is_pager;
        $is_pager++ if $is_pager;
        $next_link =~ s/(.*)(\..*)/$1_$is_pager$2/ if $is_pager;
        local $ctx->{__stash}{'next_link'} = $next_link;
        local $ctx->{__stash}{'header'} = 1;
        local $ctx->{__stash}{'repeat'} = $repeat + 1;
        local $ctx->{__stash}{'counter'} = $i;
        local $ctx->{__stash}{'pager_link'} = $pager_link;
        my $out = $builder->build( $ctx, $tokens, $cond );
        $res .= $out;
        $i++;
        for ( 1 .. $repeat ) {
            my $file_num = $_ + 1;
            my $new_link = $pager_link;
            $new_link =~ s/(.*)(\..*)/$1_$file_num$2/;
            local $ctx->{__stash}{'repeat'} = $repeat + 1;
            local $ctx->{__stash}{'counter'} = $i;
            local $ctx->{__stash}{'pager_link'} = $new_link;
            local $ctx->{__stash}{'header'} = 0;
            local $ctx->{__stash}{'footer'} = 1 if ( $i == ( $repeat + 1 ) );
            my $out = $builder->build( $ctx, $tokens, $cond );
            $res .= $out;
            $i++;
        }
    }
    $res;
}

sub _pager_current {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('current') || 0;
}

sub _if_header {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('header') || 0;
}

sub _if_footer {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('footer') || 0;
}

sub _if_prev {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('prev') || 0;
}

sub _if_next {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('next') || 0;
}

sub _if_current {
    my ( $ctx, $args, $cond ) = @_;
    my $counter = $ctx->stash('counter');
    my $current = $ctx->stash('current');
    if ( $counter == $current ) {
        return 1;
    } else {
        return 0;
    }
}

sub _pager_next {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('next_link') || '';
}

sub _pager_prev {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('prev_link') || '';
}

sub _pager_link {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('pager_link') || '';
}

sub _pager_counter {
    my ( $ctx, $args, $cond ) = @_;
    return $ctx->stash('counter') || '';
}

sub _pager_total {
    my ( $ctx, $args, $cond ) = @_;
    my $repeat = $ctx->stash('repeat');
    return $repeat if $repeat;
    my $total = MT::Template::Context::_hdlr_archive_count($ctx);
    my $template = $ctx->stash('template');
    my $tmpl_src = $template->text;
    my $limit = $1 if $tmpl_src =~ /<mt:*entries.*?limit=.*?([0-9]{1,}).*?>/is;
    $repeat = 1;
    if ( $limit < $total ) {
        $repeat = $total / $limit;
        $repeat = ceil ( $repeat );
    }
    return $repeat;
}

sub _edit_template {
    my ( $cb, $app, $param, $tmpl ) = @_;
    my $at = $param->{type};
    if ( ( $at eq 'archive' ) || ( $at eq 'index' ) ) {
        my $pointer_field = $tmpl->getElementById( 'linked_file' );
        my $nodeset = $tmpl->createElement('app:setting', { id => 'pager', label => $plugin->translate('Pager') , required => 0 });
        my $innerHTML = '<label><input name="pager" id="pager" type="checkbox" <mt:if name="pager">checked="checked"</mt:if> value="1" />';
        $innerHTML .= '<input name="pager" type="hidden" value="0" /> ' . $plugin->translate('Split this archive page') . '</label>';
        $nodeset->innerHTML( $innerHTML );
        $tmpl->insertAfter( $nodeset, $pointer_field );
    }
}

sub ceil {
    my $var = shift;
    my $a = 0;
    $a = 1 if( $var > 0 and $var != int($var) );
    return int( $var + $a );
}

1;